@@ -18,7 +18,8 @@
 
 package org.dogtagpki.server.ca.rest;
 
-import java.io.File;
+import java.io.ByteArrayInputStream;
+import java.io.ByteArrayOutputStream;
 import java.io.IOException;
 import java.net.URI;
 import java.security.Principal;
@@ -28,6 +29,7 @@ import java.util.LinkedHashMap;
 import java.util.List;
 import java.util.Locale;
 import java.util.Map;
+import java.util.Properties;
 import java.util.Vector;
 
 import javax.servlet.http.HttpServletRequest;
@@ -78,6 +80,8 @@ import com.netscape.cms.realm.PKIPrincipal;
 import com.netscape.cms.servlet.base.PKIService;
 import com.netscape.cms.servlet.profile.PolicyConstraintFactory;
 import com.netscape.cms.servlet.profile.PolicyDefaultFactory;
+import com.netscape.cmscore.base.SimpleProperties;
+import com.netscape.cmscore.base.PropConfigStore;
 
 /**
  * @author alee
@@ -164,9 +168,7 @@ public class ProfileService extends PKIService implements ProfileResource {
         return createOKResponse(infos);
     }
 
-    @Override
-    public Response retrieveProfile(String profileId) throws ProfileNotFoundException {
-        ProfileData data = null;
+    private IProfile getProfile(String profileId) throws ProfileNotFoundException {
         boolean visibleOnly = true;
 
         if (profileId == null) {
@@ -186,24 +188,11 @@ public class ProfileService extends PKIService implements ProfileResource {
             visibleOnly = false;
         }
 
-        Enumeration<String> profileIds = ps.getProfileIds();
-
-        IProfile profile = null;
-        if (profileIds != null) {
-            while (profileIds.hasMoreElements()) {
-                String id = profileIds.nextElement();
-
-                if (id.equals(profileId)) {
-
-                    try {
-                        profile = ps.getProfile(profileId);
-                    } catch (EProfileException e) {
-                        e.printStackTrace();
-                        throw new ProfileNotFoundException(profileId);
-                    }
-                    break;
-                }
-            }
+        IProfile profile;
+        try {
+            profile = ps.getProfile(profileId);
+        } catch (EProfileException e) {
+            throw new ProfileNotFoundException(profileId, "Profile not found", e);
         }
 
         if (profile == null) {
@@ -214,6 +203,14 @@ public class ProfileService extends PKIService implements ProfileResource {
             throw new ProfileNotFoundException(profileId);
         }
 
+        return profile;
+    }
+
+    @Override
+    public Response retrieveProfile(String profileId) throws ProfileNotFoundException {
+        IProfile profile = getProfile(profileId);
+
+        ProfileData data = null;
         try {
             data = createProfileData(profileId);
         } catch (EBaseException e) {
@@ -229,6 +226,19 @@ public class ProfileService extends PKIService implements ProfileResource {
         return createOKResponse(data);
     }
 
+    @Override
+    public Response retrieveProfileRaw(String profileId)
+            throws ProfileNotFoundException {
+        IProfile profile = getProfile(profileId);
+        ByteArrayOutputStream data = new ByteArrayOutputStream();
+        // add profileId and classId "virtual" properties
+        profile.getConfigStore().put("profileId", profileId);
+        profile.getConfigStore().put("classId", ps.getProfileClassId(profileId));
+        profile.getConfigStore().save(data, null);
+        return createOKResponse(data.toByteArray());
+    }
+
+
     public ProfileData createProfileData(String profileId) throws EBaseException {
 
         IProfile profile;
@@ -244,7 +254,7 @@ public class ProfileService extends PKIService implements ProfileResource {
 
         data.setAuthenticatorId(profile.getAuthenticatorId());
         data.setAuthzAcl(profile.getAuthzAcl());
-        data.setClassId(cs.getString(profileId + ".class_id"));
+        data.setClassId(ps.getProfileClassId(profileId));
         data.setDescription(profile.getDescription(getLocale(headers)));
         data.setEnabled(ps.isProfileEnable(profileId));
         data.setEnabledBy(ps.getProfileEnableBy(profileId));
@@ -379,6 +389,11 @@ public class ProfileService extends PKIService implements ProfileResource {
             throw new BadRequestException("Unable to modify profile state: Invalid Profile Id");
         }
 
+        if (action == null) {
+            CMS.debug("modifyProfileState: invalid request. action is null");
+            throw new BadRequestException("Unable to modify profile state: Missing action");
+        }
+
         if (ps == null) {
             CMS.debug("modifyProfileState: ps is null");
             throw new PKIException("Error modifying profile state.  Profile Service not available");
@@ -472,18 +487,13 @@ public class ProfileService extends PKIService implements ProfileResource {
             auditParams.put("description", data.getDescription());
             auditParams.put("visible", Boolean.toString(data.isVisible()));
 
-            String config = CMS.getConfigStore().getString("instanceRoot") + "/ca/profiles/ca/" +
-                    profileId + ".cfg";
-            File configFile = new File(config);
-            configFile.createNewFile();
             IPluginInfo info = registry.getPluginInfo("profile", data.getClassId());
 
-            profile = ps.createProfile(profileId, data.getClassId(), info.getClassName(), config);
+            profile = ps.createProfile(profileId, data.getClassId(), info.getClassName());
             profile.setName(getLocale(headers), data.getName());
             profile.setDescription(getLocale(headers), data.getDescription());
             profile.setVisible(data.isVisible());
             profile.getConfigStore().commit(false);
-            ps.createProfileConfig(profileId, data.getClassId(), config);
 
             if (profile instanceof IProfileEx) {
                 // populates profile specific plugins such as
@@ -504,6 +514,109 @@ public class ProfileService extends PKIService implements ProfileResource {
 
             return createCreatedResponse(profileData, profileData.getLink().getHref());
 
+        } catch (EBaseException e) {
+            CMS.debug("createProfile: error creating profile");
+            CMS.debug(e);
+
+            auditProfileChange(
+                    ScopeDef.SC_PROFILE_RULES,
+                    OpDef.OP_ADD,
+                    profileId,
+                    ILogger.FAILURE,
+                    auditParams);
+
+            throw new PKIException("Error in creating profile", e);
+        }
+    }
+
+    @Override
+    public Response createProfileRaw(byte[] data) {
+        if (data == null) {
+            CMS.debug("createProfileRaw: profile data is null");
+            throw new BadRequestException("Unable to create profile: Invalid profile data.");
+        }
+
+        if (ps == null) {
+            CMS.debug("createProfile: ps is null");
+            throw new PKIException("Error creating profile.  Profile Service not available");
+        }
+
+        Map<String, String> auditParams = new LinkedHashMap<String, String>();
+        String profileId = null;
+        String classId = null;
+        // First read the data into a Properties to process escaped
+        // separator characters (':', '=') in values
+        Properties properties = new Properties();
+        try {
+            // load data and read profileId and classId
+            properties.load(new ByteArrayInputStream(data));
+            profileId = properties.getProperty("profileId");
+            classId = properties.getProperty("classId");
+        } catch (IOException e) {
+            throw new BadRequestException("Could not parse raw profile data.");
+        }
+        if (profileId == null) {
+            throw new BadRequestException("Profile data did not contain profileId attribute.");
+        }
+        if (classId == null) {
+            throw new BadRequestException("Profile data did not contain classId attribute.");
+        }
+        properties.remove("profileId");
+        properties.remove("classId");
+
+        // Now copy into SimpleProperties to avoid unwanted escapes
+        // of separator characters in output
+        SimpleProperties simpleProperties = new SimpleProperties();
+        for (String k : properties.stringPropertyNames()) {
+            simpleProperties.setProperty(k, properties.getProperty(k));
+        }
+
+        try {
+            ByteArrayOutputStream out = new ByteArrayOutputStream();
+            simpleProperties.store(out, null);
+            data = out.toByteArray();  // original data sans profileId, classId
+
+            IProfile profile = ps.getProfile(profileId);
+            if (profile != null) {
+                throw new BadRequestException("Profile already exists");
+            }
+
+            auditParams.put("class_id", classId);
+
+            IPluginInfo info = registry.getPluginInfo("profile", classId);
+            String className = info.getClassName();
+
+            // create temporary profile to verify profile configuration
+            IProfile tempProfile;
+            try {
+                tempProfile = (IProfile) Class.forName(className).newInstance();
+            } catch (Exception e) {
+                throw new PKIException(
+                    "Error instantiating profile class: " + className);
+            }
+            tempProfile.setId(profileId);
+            try {
+                PropConfigStore tempConfig = new PropConfigStore(null);
+                tempConfig.load(new ByteArrayInputStream(data));
+                tempProfile.init(ps, tempConfig);
+            } catch (Exception e) {
+                throw new BadRequestException("Invalid profile data", e);
+            }
+
+            // no error thrown, proceed with profile creation
+            profile = ps.createProfile(profileId, classId, className);
+            profile.getConfigStore().commit(false);
+            profile.getConfigStore().load(new ByteArrayInputStream(data));
+            ps.disableProfile(profileId);
+
+            auditProfileChange(
+                    ScopeDef.SC_PROFILE_RULES,
+                    OpDef.OP_ADD,
+                    profileId,
+                    ILogger.SUCCESS,
+                    auditParams);
+
+            return createCreatedResponse(data, uriInfo.getAbsolutePath());
         } catch (EBaseException | IOException e) {
             CMS.debug("createProfile: error in creating profile: " + e);
             e.printStackTrace();
@@ -515,7 +628,7 @@ public class ProfileService extends PKIService implements ProfileResource {
                     ILogger.FAILURE,
                     auditParams);
 
-            throw new PKIException("Error in creating profile");
+            throw new PKIException("Error in creating profile", e);
         }
     }
 
@@ -556,6 +669,88 @@ public class ProfileService extends PKIService implements ProfileResource {
         }
     }
 
+    @Override
+    public Response modifyProfileRaw(String profileId, byte[] data) {
+        if (profileId == null) {
+            CMS.debug("modifyProfile: invalid request. profileId is null");
+            throw new BadRequestException("Unable to modify profile: Invalid Profile Id");
+        }
+
+        if (data == null) {
+            CMS.debug("modifyProfile: invalid request. data is null");
+            throw new BadRequestException("Unable to modify profile: Invalid profile data");
+        }
+
+        if (ps == null) {
+            CMS.debug("modifyProfile: ps is null");
+            throw new PKIException("Error modifying profile.  Profile Service not available");
+        }
+
+        if (ps.isProfileEnable(profileId)) {
+            throw new BadRequestException("Cannot change profile data.  Profile must be disabled");
+        }
+
+        // First read the data into a Properties to process escaped
+        // separator characters (':', '=') in values
+        Properties properties = new Properties();
+        try {
+            properties.load(new ByteArrayInputStream(data));
+        } catch (IOException e) {
+            throw new BadRequestException("Could not parse raw profile data.", e);
+        }
+        properties.remove("profileId");
+        properties.remove("classId");
+
+        // Now copy into SimpleProperties to avoid unwanted escapes
+        // of separator characters in output
+        SimpleProperties simpleProperties = new SimpleProperties();
+        for (String k : properties.stringPropertyNames()) {
+            simpleProperties.setProperty(k, properties.getProperty(k));
+        }
+
+        try {
+            IProfile profile = ps.getProfile(profileId);
+            if (profile == null) {
+                throw new ProfileNotFoundException(profileId);
+            }
+
+            ByteArrayOutputStream out = new ByteArrayOutputStream();
+            simpleProperties.store(out, null);
+            data = out.toByteArray();  // original data sans profileId, classId
+
+            // create temporary profile to verify profile configuration
+            String classId = ps.getProfileClassId(profileId);
+            String className =
+                registry.getPluginInfo("profile", classId).getClassName();
+            IProfile tempProfile;
+            try {
+                tempProfile = (IProfile) Class.forName(className).newInstance();
+            } catch (Exception e) {
+                throw new PKIException(
+                    "Error instantiating profile class: " + className);
+            }
+            tempProfile.setId(profileId);
+            try {
+                PropConfigStore tempConfig = new PropConfigStore(null);
+                tempConfig.load(new ByteArrayInputStream(data));
+                tempProfile.init(ps, tempConfig);
+            } catch (Exception e) {
+                throw new BadRequestException("Invalid profile data", e);
+            }
+
+            // no error thrown, so commit updated profile config
+            profile.getConfigStore().load(new ByteArrayInputStream(data));
+            ps.disableProfile(profileId);
+            profile.getConfigStore().commit(false);
+
+            return createOKResponse(data);
+        } catch (EBaseException | IOException e) {
+            CMS.debug("modifyProfile: error modifying profile " + profileId);
+            CMS.debug(e);
+            throw new PKIException("Error modifying profile.", e);
+        }
+    }
+
     private void changeProfileData(ProfileData data, IProfile profile) {
         String profileId = data.getId();
         if (profile == null) {
@@ -983,9 +1178,7 @@ public class ProfileService extends PKIService implements ProfileResource {
                         "`.  Profile must be disabled first.");
             }
 
-            String configFile = CMS.getConfigStore().getString("profile." + profileId + ".config");
-
-            ps.deleteProfile(profileId, configFile);
+            ps.deleteProfile(profileId);
 
             auditProfileChange(
                     ScopeDef.SC_PROFILE_RULES,
