@@ -19,7 +19,6 @@ package org.dogtagpki.server.rest;
 
 import java.math.BigInteger;
 import java.net.MalformedURLException;
-import java.net.URISyntaxException;
 import java.net.URL;
 import java.security.NoSuchAlgorithmException;
 import java.security.PublicKey;
@@ -31,7 +30,6 @@ import java.util.Random;
 import javax.servlet.http.HttpServletRequest;
 import javax.ws.rs.core.Context;
 import javax.ws.rs.core.HttpHeaders;
-import javax.ws.rs.core.MultivaluedMap;
 import javax.ws.rs.core.Request;
 import javax.ws.rs.core.UriInfo;
 
@@ -109,41 +107,42 @@ public class SystemConfigService extends PKIService implements SystemConfigResou
         instanceRoot = cs.getString("instanceRoot");
     }
 
-    /* (non-Javadoc)
-     * @see com.netscape.cms.servlet.csadmin.SystemConfigurationResource#configure(javax.ws.rs.core.MultivaluedMap)
-     */
-    @Override
-    public ConfigurationResponse configure(MultivaluedMap<String, String> form) throws URISyntaxException {
-        ConfigurationRequest data = new ConfigurationRequest(form);
-        return configure(data);
-    }
-
     /* (non-Javadoc)
      * @see com.netscape.cms.servlet.csadmin.SystemConfigurationResource#configure(com.netscape.cms.servlet.csadmin.data.ConfigurationData)
      */
     @Override
-    public ConfigurationResponse configure(ConfigurationRequest request) {
+    public ConfigurationResponse configure(ConfigurationRequest request) throws Exception {
+
+        CMS.debug("SystemConfigService: configure()");
+
         try {
             ConfigurationResponse response = new ConfigurationResponse();
             configure(request, response);
             return response;
 
-        } catch (Throwable t) {
-            CMS.debug(t);
-            throw t;
+        } catch (PKIException e) { // normal responses
+            CMS.debug(e.getMessage()); // log the response
+            throw e;
+
+        } catch (Exception e) { // unexpected exceptions
+            CMS.debug(e); // show stack trace for troubleshooting
+            throw e;
+
+        } catch (Error e) { // system errors
+            CMS.debug(e); // show stack trace for troubleshooting
+            throw e;
         }
     }
 
-    public void configure(ConfigurationRequest data, ConfigurationResponse response) {
+    public void configure(ConfigurationRequest data, ConfigurationResponse response) throws Exception {
+
 
         if (csState.equals("1")) {
             throw new BadRequestException("System is already configured");
         }
 
-        CMS.debug("SystemConfigService(): configure() called");
-        CMS.debug(data.toString());
-
-        validateData(data);
+        CMS.debug("SystemConfigService: request: " + data);
+        validateRequest(data);
 
         Collection<String> certList = getCertList(data);
 
@@ -174,7 +173,7 @@ public class SystemConfigService extends PKIService implements SystemConfigResou
             cs.commit(false);
         } catch (EBaseException e) {
             CMS.debug(e);
-            throw new PKIException("Unable to commit config parameters to file");
+            throw new PKIException("Unable to commit config parameters to file", e);
         }
         initializeDatabase(data);
 
@@ -200,8 +199,8 @@ public class SystemConfigService extends PKIService implements SystemConfigResou
                 ConfigurationUtils.setCertPermissions(cert.getCertTag());
                 CMS.debug("Processed '" + cert.getCertTag() + "' certificate.");
             } catch (Exception e) {
-                e.printStackTrace();
-                throw new PKIException("Error in configuring system certificates" + e);
+                CMS.debug(e);
+                throw new PKIException("Error in configuring system certificates" + e, e);
             }
             if (ret != 0) {
                 throw new PKIException("Error in configuring system certificates");
@@ -222,28 +221,8 @@ public class SystemConfigService extends PKIService implements SystemConfigResou
         // Done Panel
         // Create or update security domain
         CMS.debug("=== Done Panel ===");
-        try {
-            String securityDomainType = data.getSecurityDomainType();
-            if (securityDomainType.equals(ConfigurationRequest.NEW_DOMAIN)) {
-                ConfigurationUtils.createSecurityDomain();
-            } else {
-                ConfigurationUtils.updateSecurityDomain();
-            }
-            cs.putString("service.securityDomainPort", CMS.getAgentPort());
-            cs.putString("securitydomain.store", "ldap");
-            cs.commit(false);
-        } catch (Exception e) {
-            e.printStackTrace();
-            throw new PKIException("Error while updating security domain: " + e);
-        }
-
-        try {
-            if (!data.getSharedDB()) ConfigurationUtils.setupDBUser();
-        } catch (Exception e) {
-            CMS.debug(e);
-            throw new PKIException("Errors in creating or updating dbuser: " + e);
-        }
-
+        setupSecurityDomain(data);
+        setupDBUser(data);
         finalizeConfiguration(data);
 
         cs.putInteger("cs.state", 1);
@@ -254,8 +233,8 @@ public class SystemConfigService extends PKIService implements SystemConfigResou
         try {
             ConfigurationUtils.removePreopConfigEntries();
         } catch (EBaseException e) {
-            e.printStackTrace();
-            throw new PKIException("Errors when removing preop config entries: " + e);
+            CMS.debug(e);
+            throw new PKIException("Errors when removing preop config entries: " + e, e);
         }
 
         // Create an empty file that designates the fact that although
@@ -268,6 +247,46 @@ public class SystemConfigService extends PKIService implements SystemConfigResou
         response.setStatus(SUCCESS);
     }
 
+    private void setupDBUser(ConfigurationRequest data) {
+        try {
+            if (!data.getSharedDB()) ConfigurationUtils.setupDBUser();
+        } catch (Exception e) {
+            CMS.debug(e);
+            throw new PKIException("Errors in creating or updating dbuser: " + e);
+        }
+    }
+
+    private void setupSecurityDomain(ConfigurationRequest data) {
+        try {
+            String securityDomainType = data.getSecurityDomainType();
+            if (securityDomainType.equals(ConfigurationRequest.NEW_DOMAIN)) {
+                CMS.debug("Creating new security domain");
+                ConfigurationUtils.createSecurityDomain();
+            } else if (securityDomainType.equals(ConfigurationRequest.NEW_SUBDOMAIN)) {
+                CMS.debug("Creating subordinate CA security domain");
+
+                // switch out security domain parameters from issuing CA security domain
+                // to subordinate CA hosted security domain
+                cs.putString("securitydomain.name", data.getSubordinateSecurityDomainName());
+                cs.putString("securitydomain.host", CMS.getEENonSSLHost());
+                cs.putString("securitydomain.httpport", CMS.getEENonSSLPort());
+                cs.putString("securitydomain.httpsagentport", CMS.getAgentPort());
+                cs.putString("securitydomain.httpseeport", CMS.getEESSLPort());
+                cs.putString("securitydomain.httpsadminport", CMS.getAdminPort());
+                ConfigurationUtils.createSecurityDomain();
+            } else {
+                CMS.debug("Updating existing security domain");
+                ConfigurationUtils.updateSecurityDomain();
+            }
+            cs.putString("service.securityDomainPort", CMS.getAgentPort());
+            cs.putString("securitydomain.store", "ldap");
+            cs.commit(false);
+        } catch (Exception e) {
+            e.printStackTrace();
+            throw new PKIException("Error while updating security domain: " + e);
+        }
+    }
+
     public Collection<String> getCertList(ConfigurationRequest request) {
 
         Collection<String> certList = new ArrayList<String>();
@@ -309,6 +328,14 @@ public class SystemConfigService extends PKIService implements SystemConfigResou
                     if (systemCert.getTag().equals(tag)) {
                         certData = systemCert;
                         CMS.debug("Found data for '" + tag + "'");
+                        if (tag.equals("signing") &&
+                                certData.getReqExtOID() != null &&
+                                certData.getReqExtData() != null) {
+                            CMS.debug("SystemConfigService:processCerts: adding request extension to config");
+                            cs.putString("preop.cert.signing.ext.oid", certData.getReqExtOID());
+                            cs.putString("preop.cert.signing.ext.data", certData.getReqExtData());
+                            cs.putBoolean("preop.cert.signing.ext.critical", certData.getReqExtCritical());
+                        }
                         break;
                     }
                 }
@@ -318,6 +345,7 @@ public class SystemConfigService extends PKIService implements SystemConfigResou
                     throw new BadRequestException("No data for '" + tag + "' was found!");
                 }
 
+                String tokenName = certData.getToken() != null ? certData.getToken() : token;
                 if (request.getStandAlone() && request.getStepTwo()) {
                     // Stand-alone PKI (Step 2)
                     if (tag.equals("external_signing")) {
@@ -328,7 +356,6 @@ public class SystemConfigService extends PKIService implements SystemConfigResou
 
                             if (request.getIssuingCA().equals("External CA")) {
                                 String nickname = certData.getNickname() != null ? certData.getNickname() : "caSigningCert External CA";
-                                String tokenName = certData.getToken() != null ? certData.getToken() : token;
                                 Cert cert = new Cert(tokenName, nickname, tag);
                                 ConfigurationUtils.setExternalCACert(b64, csSubsystem, cs, cert);
 
@@ -360,7 +387,7 @@ public class SystemConfigService extends PKIService implements SystemConfigResou
                     updateConfiguration(request, certData, "subsystem");
 
                     // get parameters needed for cloning
-                    updateCloneConfiguration(certData, "subsystem");
+                    updateCloneConfiguration(certData, "subsystem", tokenName);
                     continue;
                 }
 
@@ -383,6 +410,17 @@ public class SystemConfigService extends PKIService implements SystemConfigResou
                 cs.putString("preop.cert." + tag + ".nickname", nickname);
                 cs.putString("preop.cert." + tag + ".dn", dn);
 
+                // support injecting SAN into server cert
+                if ( tag.equals("sslserver") && certData.getServerCertSAN() != null) {
+                    CMS.debug("updateConfiguration(): san_server_cert found");
+                    cs.putString("service.injectSAN", "true");
+                    cs.putString("service.sslserver.san", certData.getServerCertSAN());
+                } else {
+                    if ( tag.equals("sslserver"))
+                        CMS.debug("SystemConfigService:processCerts(): san_server_cert not found for tag sslserver");
+                }
+                cs.commit(false);
+
                 if (!request.getStepTwo()) {
                     if (keytype.equals("ecc")) {
                         String curvename = certData.getKeyCurveName() != null ?
@@ -401,7 +439,6 @@ public class SystemConfigService extends PKIService implements SystemConfigResou
                     CMS.debug("configure(): step two selected.  keys will not be generated for '" + tag + "'");
                 }
 
-                String tokenName = certData.getToken() != null ? certData.getToken() : token;
                 Cert cert = new Cert(tokenName, nickname, tag);
                 cert.setDN(dn);
                 cert.setSubsystem(cs.getString("preop.cert." + tag + ".subsystem"));
@@ -491,11 +528,16 @@ public class SystemConfigService extends PKIService implements SystemConfigResou
         }
     }
 
-    private void updateCloneConfiguration(SystemCertData cdata, String tag) throws NotInitializedException,
+    private void updateCloneConfiguration(SystemCertData cdata, String tag, String tokenName) throws NotInitializedException,
             ObjectNotFoundException, TokenException {
         // TODO - some of these parameters may only be valid for RSA
         CryptoManager cryptoManager = CryptoManager.getInstance();
-        X509Certificate cert = cryptoManager.findCertByNickname(cdata.getNickname());
+        if (!tokenName.isEmpty())
+            CMS.debug("SystemConfigService:updateCloneConfiguration: tokenName=" + tokenName);
+        else
+            CMS.debug("SystemConfigService:updateCloneConfiguration: tokenName empty; using internal");
+
+        X509Certificate cert = cryptoManager.findCertByNickname(!tokenName.isEmpty()? tokenName + ":" + cdata.getNickname() :  cdata.getNickname());
         PublicKey pubk = cert.getPublicKey();
         byte[] exponent = CryptoUtil.getPublicExponent(pubk);
         byte[] modulus = CryptoUtil.getModulus(pubk);
@@ -603,7 +645,7 @@ public class SystemConfigService extends PKIService implements SystemConfigResou
         cs.putString("internaldb.database", data.getDatabase());
         cs.putString("internaldb.basedn", data.getBaseDN());
         cs.putString("internaldb.ldapauth.bindDN", data.getBindDN());
-        cs.putBoolean("internaldb.ldapconn.secureConn", data.getSecureConn().equals("on"));
+        cs.putBoolean("internaldb.ldapconn.secureConn", data.getSecureConn().equals("true"));
         cs.putString("preop.database.removeData", data.getRemoveData());
         cs.putBoolean("preop.database.createNewDB", data.getCreateNewDB());
         cs.putBoolean("preop.database.setupReplication", data.getSetupReplication());
@@ -646,7 +688,7 @@ public class SystemConfigService extends PKIService implements SystemConfigResou
             cs.putString("internaldb.ldapconn.cloneReplicationPort", cloneReplicationPort);
 
             String replicationSecurity = data.getReplicationSecurity();
-            if ((cloneReplicationPort == data.getDsPort()) && (data.getSecureConn().equals("on"))) {
+            if ((cloneReplicationPort == data.getDsPort()) && (data.getSecureConn().equals("true"))) {
                 replicationSecurity = "SSL";
             } else if (replicationSecurity == null) {
                 replicationSecurity = "None";
@@ -658,7 +700,13 @@ public class SystemConfigService extends PKIService implements SystemConfigResou
 
         try {
             /* BZ 430745 create password for replication manager */
-            String replicationpwd = Integer.toString(new Random().nextInt());
+            // use user-provided password if specified
+            String replicationPassword = data.getReplicationPassword();
+
+            if (StringUtils.isEmpty(replicationPassword)) {
+                // generate random password
+                replicationPassword = Integer.toString(new Random().nextInt());
+            }
 
             IConfigStore psStore = null;
             String passwordFile = null;
@@ -666,14 +714,14 @@ public class SystemConfigService extends PKIService implements SystemConfigResou
             psStore = CMS.createFileConfigStore(passwordFile);
             psStore.putString("internaldb", data.getBindpwd());
             if (data.getSetupReplication()) {
-                psStore.putString("replicationdb", replicationpwd);
+                psStore.putString("replicationdb", replicationPassword);
             }
             psStore.commit(false);
 
             if (!data.getStepTwo()) {
                 ConfigurationUtils.populateDB();
 
-                cs.putString("preop.internaldb.replicationpwd", replicationpwd);
+                cs.putString("preop.internaldb.replicationpwd", replicationPassword);
                 cs.putString("preop.database.removeData", "false");
                 if (data.getSharedDB()) {
                     cs.putString("preop.internaldb.dbuser", data.getSharedDBUserDN());
@@ -728,10 +776,10 @@ public class SystemConfigService extends PKIService implements SystemConfigResou
                     CMS.debug("local CA selected");
                     url = url.substring(url.indexOf("https"));
                     cs.putString("preop.ca.url", url);
-
                     URL urlx = new URL(url);
                     String host = urlx.getHost();
                     int port = urlx.getPort();
+
                     int admin_port = ConfigurationUtils.getPortFromSecurityDomain(domainXML,
                             host, port, "CA", "SecurePort", "SecureAdminPort");
 
@@ -755,7 +803,7 @@ public class SystemConfigService extends PKIService implements SystemConfigResou
         }
     }
 
-    private void getCloningData(ConfigurationRequest data, Collection<String> certList, String token, String domainXML) {
+    private void configureClone(ConfigurationRequest data, Collection<String> certList, String token, String domainXML) throws Exception {
         for (String tag : certList) {
             if (tag.equals("sslserver")) {
                 cs.putBoolean("preop.cert." + tag + ".enable", true);
@@ -765,63 +813,45 @@ public class SystemConfigService extends PKIService implements SystemConfigResou
         }
 
         String cloneUri = data.getCloneUri();
-        URL url = null;
-        try {
-            url = new URL(cloneUri);
-        } catch (MalformedURLException e) {
-            // should not reach here as this check is done in validate()
-        }
+        URL url = new URL(cloneUri);
         String masterHost = url.getHost();
         int masterPort = url.getPort();
 
-        // check and store cloneURI information
-        boolean validCloneUri;
-        try {
-            validCloneUri = ConfigurationUtils.isValidCloneURI(domainXML, masterHost, masterPort);
-        } catch (Exception e) {
-            e.printStackTrace();
-            throw new PKIException("Error in determining whether clone URI is valid");
-        }
+        CMS.debug("SystemConfigService: validate clone URI: " + url);
+        boolean validCloneUri = ConfigurationUtils.isValidCloneURI(domainXML, masterHost, masterPort);
 
         if (!validCloneUri) {
             throw new BadRequestException(
-                    "Invalid clone URI provided.  Does not match the available subsystems in the security domain");
+                    "Clone URI does not match available subsystems: " + url);
         }
 
         if (csType.equals("CA")) {
-            try {
-                int masterAdminPort = ConfigurationUtils.getPortFromSecurityDomain(domainXML,
-                        masterHost, masterPort, "CA", "SecurePort", "SecureAdminPort");
-                ConfigurationUtils.importCertChain(masterHost, masterAdminPort, "/ca/admin/ca/getCertChain",
-                        "clone");
-            } catch (Exception e) {
-                e.printStackTrace();
-                throw new PKIException("Failed to import certificate chain from master" + e);
-            }
+            CMS.debug("SystemConfigService: import certificate chain from master");
+            int masterAdminPort = ConfigurationUtils.getPortFromSecurityDomain(domainXML,
+                    masterHost, masterPort, "CA", "SecurePort", "SecureAdminPort");
+            ConfigurationUtils.importCertChain(masterHost, masterAdminPort,
+                    "/ca/admin/ca/getCertChain", "clone");
         }
 
-        try {
-            ConfigurationUtils.getConfigEntriesFromMaster();
-        } catch (Exception e) {
-            e.printStackTrace();
-            throw new PKIException("Failed to obtain configuration entries from the master for cloning " + e);
-        }
+        CMS.debug("SystemConfigService: get configuration entries from master");
+        ConfigurationUtils.getConfigEntriesFromMaster();
 
-        // restore certs from P12 file
         if (token.equals(ConfigurationRequest.TOKEN_DEFAULT)) {
+            CMS.debug("SystemConfigService: restore certificates from P12 file");
             String p12File = data.getP12File();
             String p12Pass = data.getP12Password();
-            try {
-                ConfigurationUtils.restoreCertsFromP12(p12File, p12Pass);
-            } catch (Exception e) {
-                e.printStackTrace();
-                throw new PKIException("Failed to restore certificates from p12 file" + e);
-            }
+            ConfigurationUtils.restoreCertsFromP12(p12File, p12Pass);
+
+        } else {
+            CMS.debug("SystemConfigService: import certificates from HSM and set permission");
+            ConfigurationUtils.importAndSetCertPermissionsFromHSM();
         }
 
+        CMS.debug("SystemConfigService: verify certificates");
         boolean cloneReady = ConfigurationUtils.isCertdbCloned();
+
         if (!cloneReady) {
-            CMS.debug("clone does not have all the certificates.");
+            CMS.debug("SystemConfigService: clone does not have all the certificates.");
             throw new PKIException("Clone does not have all the required certificates");
         }
     }
@@ -832,83 +862,113 @@ public class SystemConfigService extends PKIService implements SystemConfigResou
 
         String securityDomainType = data.getSecurityDomainType();
         String securityDomainName = data.getSecurityDomainName();
-        String securityDomainURL = data.getSecurityDomainUri();
 
         if (securityDomainType.equals(ConfigurationRequest.NEW_DOMAIN)) {
-            CMS.debug("Creating new security domain");
-            cs.putString("preop.securitydomain.select", "new");
-            cs.putString("securitydomain.select", "new");
-            cs.putString("preop.securitydomain.name", securityDomainName);
-            cs.putString("securitydomain.name", securityDomainName);
-            cs.putString("securitydomain.host", CMS.getEENonSSLHost());
-            cs.putString("securitydomain.httpport", CMS.getEENonSSLPort());
-            cs.putString("securitydomain.httpsagentport", CMS.getAgentPort());
-            cs.putString("securitydomain.httpseeport", CMS.getEESSLPort());
-            cs.putString("securitydomain.httpsadminport", CMS.getAdminPort());
-            // Stand-alone PKI (Step 1)
-            if (data.getStandAlone()) {
-                cs.putString("preop.cert.subsystem.type", "remote");
-            } else {
-                cs.putString("preop.cert.subsystem.type", "local");
-            }
-            cs.putString("preop.cert.subsystem.profile", "subsystemCert.profile");
-
+            configureNewSecurityDomain(data, securityDomainName);
+        } else if (securityDomainType.equals(ConfigurationRequest.NEW_SUBDOMAIN)){
+            CMS.debug("Configuring new subordinate root CA");
+            configureNewSecurityDomain(data, data.getSubordinateSecurityDomainName());
+            String securityDomainURL = data.getSecurityDomainUri();
+            domainXML = logIntoSecurityDomain(data, securityDomainURL);
         } else {
             CMS.debug("Joining existing security domain");
             cs.putString("preop.securitydomain.select", "existing");
             cs.putString("securitydomain.select", "existing");
             cs.putString("preop.cert.subsystem.type", "remote");
             cs.putString("preop.cert.subsystem.profile", "caInternalAuthSubsystemCert");
-
-            CMS.debug("Getting certificate chain");
-            // contact and log onto security domain
-            URL secdomainURL;
-            String host;
-            int port;
-            try {
-                secdomainURL = new URL(securityDomainURL);
-                host = secdomainURL.getHost();
-                port = secdomainURL.getPort();
-                cs.putString("securitydomain.host", host);
-                cs.putInteger("securitydomain.httpsadminport",port);
-                ConfigurationUtils.importCertChain(host, port, "/ca/admin/ca/getCertChain", "securitydomain");
-            } catch (Exception e) {
-                e.printStackTrace();
-                throw new PKIException("Failed to import certificate chain from security domain master: " + e);
-            }
-
-            CMS.debug("Getting install token");
-            // log onto security domain and get token
-            String user = data.getSecurityDomainUser();
-            String pass = data.getSecurityDomainPassword();
-            String installToken;
-            try {
-                installToken = ConfigurationUtils.getInstallToken(host, port, user, pass);
-            } catch (Exception e) {
-                e.printStackTrace();
-                throw new PKIException("Failed to obtain installation token from security domain: " + e);
-            }
-
-            if (installToken == null) {
-                CMS.debug("Install token is null");
-                throw new PKIException("Failed to obtain installation token from security domain");
-            }
-            CMS.setConfigSDSessionId(installToken);
-
-            CMS.debug("Getting domain XML");
-            try {
-                domainXML = ConfigurationUtils.getDomainXML(host, port, true);
-                ConfigurationUtils.getSecurityDomainPorts(domainXML, host, port);
-            } catch (Exception e) {
-                e.printStackTrace();
-                throw new PKIException("Failed to obtain security domain decriptor from security domain master: " + e);
-            }
+            String securityDomainURL = data.getSecurityDomainUri();
+            domainXML = logIntoSecurityDomain(data, securityDomainURL);
         }
         return domainXML;
     }
 
+    private void configureNewSecurityDomain(ConfigurationRequest data, String securityDomainName) {
+        CMS.debug("Creating new security domain");
+        cs.putString("preop.securitydomain.select", "new");
+        cs.putString("securitydomain.select", "new");
+        cs.putString("preop.securitydomain.name", securityDomainName);
+        cs.putString("securitydomain.name", securityDomainName);
+        cs.putString("securitydomain.host", CMS.getEENonSSLHost());
+        cs.putString("securitydomain.httpport", CMS.getEENonSSLPort());
+        cs.putString("securitydomain.httpsagentport", CMS.getAgentPort());
+        cs.putString("securitydomain.httpseeport", CMS.getEESSLPort());
+        cs.putString("securitydomain.httpsadminport", CMS.getAdminPort());
+        // Stand-alone PKI (Step 1)
+        if (data.getStandAlone()) {
+            cs.putString("preop.cert.subsystem.type", "remote");
+        } else {
+            cs.putString("preop.cert.subsystem.type", "local");
+        }
+        cs.putString("preop.cert.subsystem.profile", "subsystemCert.profile");
+    }
+
+    private String logIntoSecurityDomain(ConfigurationRequest data, String securityDomainURL) {
+        URL secdomainURL;
+        String host;
+        int port;
+        try {
+            CMS.debug("Resolving security domain URL" + securityDomainURL);
+            secdomainURL = new URL(securityDomainURL);
+            host = secdomainURL.getHost();
+            port = secdomainURL.getPort();
+            cs.putString("securitydomain.host", host);
+            cs.putInteger("securitydomain.httpsadminport",port);
+        } catch (Exception e) {
+            CMS.debug(e);
+            throw new PKIException("Failed to resolve security domain URL", e);
+        }
+
+        getCertChainFromSecurityDomain(host, port);
+        getInstallToken(data, host, port);
+
+        return getDomainXML(host, port);
+    }
+
+    private String getDomainXML(String host, int port) {
+        CMS.debug("Getting domain XML");
+        String domainXML = null;
+        try {
+            domainXML = ConfigurationUtils.getDomainXML(host, port, true);
+            ConfigurationUtils.getSecurityDomainPorts(domainXML, host, port);
+        } catch (Exception e) {
+            e.printStackTrace();
+            throw new PKIException("Failed to obtain security domain decriptor from security domain master: " + e);
+        }
+        return domainXML;
+    }
+
+    private void getCertChainFromSecurityDomain(String host, int port) {
+        CMS.debug("Getting security domain cert chain");
+        try {
+            ConfigurationUtils.importCertChain(host, port, "/ca/admin/ca/getCertChain", "securitydomain");
+        } catch (Exception e) {
+            e.printStackTrace();
+            throw new PKIException("Failed to import certificate chain from security domain master: " + e);
+        }
+    }
+
+    private void getInstallToken(ConfigurationRequest data, String host, int port) {
+        CMS.debug("Getting install token");
+        // log onto security domain and get token
+        String user = data.getSecurityDomainUser();
+        String pass = data.getSecurityDomainPassword();
+        String installToken;
+        try {
+            installToken = ConfigurationUtils.getInstallToken(host, port, user, pass);
+        } catch (Exception e) {
+            CMS.debug(e);
+            throw new PKIException("Failed to obtain installation token from security domain: " + e, e);
+        }
+
+        if (installToken == null) {
+            CMS.debug("Install token is null");
+            throw new PKIException("Failed to obtain installation token from security domain");
+        }
+        CMS.setConfigSDSessionId(installToken);
+    }
+
     public void configureSubsystem(ConfigurationRequest request,
-            Collection<String> certList, String token, String domainXML) {
+            Collection<String> certList, String token, String domainXML) throws Exception {
 
         cs.putString("preop.subsystem.name", request.getSubsystemName());
 
@@ -920,7 +980,7 @@ public class SystemConfigService extends PKIService implements SystemConfigResou
         } else {
             cs.putString("preop.subsystem.select", "clone");
             cs.putString("subsystem.select", "Clone");
-            getCloningData(request, certList, token, domainXML);
+            configureClone(request, certList, token, domainXML);
         }
     }
 
@@ -946,22 +1006,15 @@ public class SystemConfigService extends PKIService implements SystemConfigResou
         }
     }
 
-    private void validateData(ConfigurationRequest data) {
-        // get required info from CS.cfg
-        String preopPin;
-        try {
-            preopPin = cs.getString("preop.pin");
-        } catch (Exception e) {
-            CMS.debug("validateData: Failed to get required config form CS.cfg");
-            e.printStackTrace();
-            throw new PKIException("Unable to retrieve required configuration from configuration files");
-        }
+    private void validateRequest(ConfigurationRequest data) throws Exception {
 
-        // get the preop pin and validate it
+        // validate installation pin
         String pin = data.getPin();
         if (pin == null) {
             throw new BadRequestException("No preop pin provided");
         }
+
+        String preopPin = cs.getString("preop.pin");
         if (!preopPin.equals(pin)) {
             throw new BadRequestException("Incorrect pin provided");
         }
@@ -993,7 +1046,9 @@ public class SystemConfigService extends PKIService implements SystemConfigResou
             if (data.getSecurityDomainName() == null) {
                 throw new BadRequestException("Security Domain Name is not provided");
             }
-        } else if (domainType.equals(ConfigurationRequest.EXISTING_DOMAIN)) {
+
+        } else if (domainType.equals(ConfigurationRequest.EXISTING_DOMAIN) ||
+                   domainType.equals(ConfigurationRequest.NEW_SUBDOMAIN)) {
             if (data.getStandAlone()) {
                 throw new BadRequestException("Existing security domains are not valid for stand-alone PKI subsytems");
             }
@@ -1004,11 +1059,11 @@ public class SystemConfigService extends PKIService implements SystemConfigResou
             }
 
             try {
-                @SuppressWarnings("unused")
-                URL admin_u = new URL(domainURI);  // check for invalid URL
+                new URL(domainURI);
             } catch (MalformedURLException e) {
-                throw new BadRequestException("Invalid security domain URI");
+                throw new BadRequestException("Invalid security domain URI: " + domainURI, e);
             }
+
             if ((data.getSecurityDomainUser() == null) || (data.getSecurityDomainPassword() == null)) {
                 throw new BadRequestException("Security domain user or password not provided");
             }
@@ -1017,6 +1072,13 @@ public class SystemConfigService extends PKIService implements SystemConfigResou
             throw new BadRequestException("Invalid security domain URI provided");
         }
 
+        // validate subordinate CA security domain settings
+        if (domainType.equals(ConfigurationRequest.NEW_SUBDOMAIN)) {
+            if (StringUtils.isEmpty(data.getSubordinateSecurityDomainName())) {
+                throw new BadRequestException("Subordinate CA security domain name not provided");
+            }
+        }
+
         if ((data.getSubsystemName() == null) || (data.getSubsystemName().length() ==0)) {
             throw new BadRequestException("Invalid or no subsystem name provided");
         }
@@ -1027,11 +1089,13 @@ public class SystemConfigService extends PKIService implements SystemConfigResou
                 throw new BadRequestException("Clone selected, but no clone URI provided");
             }
             try {
-                @SuppressWarnings("unused")
-                URL url = new URL(cloneUri); // check for invalid URL
+                URL url = new URL(cloneUri);
                 // confirm protocol is https
+                if (!"https".equals(url.getProtocol())) {
+                    throw new BadRequestException("Clone URI must use HTTPS protocol: " + cloneUri);
+                }
             } catch (MalformedURLException e) {
-                throw new BadRequestException("Invalid clone URI");
+                throw new BadRequestException("Invalid clone URI: " + cloneUri, e);
             }
 
             if (data.getToken().equals(ConfigurationRequest.TOKEN_DEFAULT)) {
@@ -1042,7 +1106,16 @@ public class SystemConfigService extends PKIService implements SystemConfigResou
                 if (data.getP12Password() == null) {
                     throw new BadRequestException("P12 password not provided");
                 }
+            } else {
+                if (data.getP12File() != null) {
+                    throw new BadRequestException("P12 filename should not be provided since HSM clones must share their HSM master's private keys");
+                }
+
+                if (data.getP12Password() != null) {
+                    throw new BadRequestException("P12 password should not be provided since HSM clones must share their HSM master's private keys");
+                }
             }
+
         } else {
             data.setClone("false");
         }
@@ -1055,7 +1128,7 @@ public class SystemConfigService extends PKIService implements SystemConfigResou
         try {
             Integer.parseInt(data.getDsPort());  // check for errors
         } catch (NumberFormatException e) {
-            throw new BadRequestException("Internal database port is invalid");
+            throw new BadRequestException("Internal database port is invalid: " + data.getDsPort(), e);
         }
 
         String basedn = data.getBaseDN();
@@ -1083,7 +1156,7 @@ public class SystemConfigService extends PKIService implements SystemConfigResou
             try {
                 Integer.parseInt(masterReplicationPort); // check for errors
             } catch (NumberFormatException e) {
-                throw new BadRequestException("Master replication port is invalid");
+                throw new BadRequestException("Master replication port is invalid: " + masterReplicationPort, e);
             }
         }
 
@@ -1091,8 +1164,8 @@ public class SystemConfigService extends PKIService implements SystemConfigResou
         if (cloneReplicationPort != null && cloneReplicationPort.length() > 0) {
             try {
                 Integer.parseInt(cloneReplicationPort); // check for errors
-            } catch (Exception e) {
-                throw new BadRequestException("Clone replication port is invalid");
+            } catch (NumberFormatException e) {
+                throw new BadRequestException("Clone replication port is invalid: " + cloneReplicationPort, e);
             }
         }
 
@@ -1103,6 +1176,10 @@ public class SystemConfigService extends PKIService implements SystemConfigResou
         }
 
         if ((data.getBackupKeys() != null) && data.getBackupKeys().equals("true")) {
+            if (! data.getToken().equals(ConfigurationRequest.TOKEN_DEFAULT)) {
+                throw new BadRequestException("HSMs cannot publish private keys to PKCS #12 files");
+            }
+
             if ((data.getBackupFile() == null) || (data.getBackupFile().length()<=0)) {
                 //TODO: also check for valid path, perhaps by touching file there
                 throw new BadRequestException("Invalid key backup file name");
@@ -1199,7 +1276,7 @@ public class SystemConfigService extends PKIService implements SystemConfigResou
             try {
                 Integer.parseInt(data.getAuthdbPort()); // check for errors
             } catch (NumberFormatException e) {
-                throw new BadRequestException("Authdb port is invalid");
+                throw new BadRequestException("Authentication Database port is invalid: " + data.getAuthdbPort(), e);
             }
 
             // TODO check connection with authdb
